﻿// Copyright (c) 2020, Identix identix@identix.com.br
// 
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

using System;
using System.IO.Ports;
using System.Threading;

namespace Identix.SerialSDK.Examples.SerialConfiguration
{
    class Program
    {
        private static SerialPort serialPort = null;
        private static string[] readSeparator = { ((char)10).ToString() };
        private static string reportSeparator = ";";
        private static string epc = "";
        private static double rssi = 0;
        private static bool configMode = false;

        static void Main(string[] args)
        {
            try
            {
                Console.Write("Inform the COM port number: ");
                string comPort = Console.ReadLine();
                Console.WriteLine("COM{0}", comPort);

                // Configure serial port
                SetSerialPort("COM" + comPort);

                // Open serial port
                OpenSerialPort();

                // Set configuration
                Console.WriteLine("Configuring, wait...");
                SetConfig("Inventory", "SS");    // Reader mode Dual Target
                SetConfig("TXPower", "10");     // Tx Power

                // Other examples:
                // The complete list can be found in the archive Identix-Pad.cfg
                /*
                SetConfig("Inventory", "D");        // Reader mode Dual Target
                SetConfig("Inventory", "S");        // Reader mode Single Target
                SetConfig("Inventory", "SS");       // Reader mode Single Target with Suppression
                SetConfig("Session", "1");          // Session mode
                SetConfig("TXPower", "10");         // Tx Power
                SetConfig("DecodeSGTIN96", "False");// SGTIN96 Decode                                           
                */

                // Start inventory
                Start();

                Thread.Sleep(2000);

                // Stop inventory
                Stop();

                Console.WriteLine("Press any key to continue...");
                Console.ReadLine();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Unexpected error: " + ex.Message);

                Console.WriteLine("Press any key to continue...");
                Console.ReadLine();
            }
        }

        /// <summary>
        /// Configure serial port
        /// </summary>
        /// <param name="comPort"></param>
        private static void SetSerialPort(string comPort)
        {
            if (serialPort == null)
            {
                serialPort = new SerialPort(comPort, 115200, Parity.None, 8, StopBits.One);

                serialPort.DataReceived += new SerialDataReceivedEventHandler(PortReceivedData);
            }
        }

        /// <summary>
        /// Open serial port
        /// </summary>
        /// <returns></returns>
        private static bool OpenSerialPort()
        {
            try
            {
                if (!serialPort.IsOpen)
                    serialPort.Open();

                serialPort.DiscardInBuffer();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Could not open serial port connection. Exception: " + ex.Message);

                return false;
            }

            return true;
        }

        /// <summary>
        /// Send command to start inventory
        /// </summary>
        private static void Start()
        {
            if (!serialPort.IsOpen)
                return;

            try
            {
                serialPort.Write("1");
                Thread.Sleep(500);
            }
            catch { }
        }

        /// <summary>
        /// Send command to stop inventory
        /// </summary>
        private static void Stop()
        {
            if (!serialPort.IsOpen)
                return;

            try
            {
                serialPort.Write("0");
                Thread.Sleep(500);
            }
            catch { }
        }

        /// <summary>
        /// Reveived data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void PortReceivedData(object sender, SerialDataReceivedEventArgs e)
        {
            if (configMode) return;

            string buffer = serialPort.ReadExisting();

            string[] result = buffer.Split(readSeparator, StringSplitOptions.RemoveEmptyEntries);

            foreach (string s in result)
            {
                if (s.Contains("Inventory started"))
                {
                    Console.WriteLine("Inventory started!");
                    return;
                }
                else if (s.Contains("Inventory stopped"))
                {
                    Console.WriteLine("Inventory stopped!");
                    return;
                }
                else
                {
                    if (s != "\r")
                    {
                        if (s.Contains(";") && s.Length > 10)
                        {
                            string[] sData = s.Split(Convert.ToChar(reportSeparator));

                            epc = sData[0];
                            rssi = Convert.ToDouble(sData[1]);

                            Console.WriteLine("EPC: " + epc.ToUpper() + ", RSSI: " + rssi);
                            epc = ""; rssi = 0;
                        }
                        else if (s.Contains(reportSeparator))
                        {
                            rssi = Convert.ToDouble(s.Replace(reportSeparator, ""));

                            if (string.IsNullOrEmpty(epc)) return;

                            Console.WriteLine("EPC: " + epc.ToUpper() + ", RSSI: " + rssi);
                            epc = ""; rssi = 0;
                        }
                        else
                        {
                            epc = s;
                        }

                        Thread.Sleep(5);
                    }

                    return;
                }
            }
        }

        /// <summary>
        /// Set configuration
        /// </summary>
        /// <param name="cfg"></param>
        /// <param name="value"></param>
        private static void SetConfig(String cfg, String value)
        {
            if (!serialPort.IsOpen) return;

            configMode = true;

            try
            {
                // Command start configuration mode
                serialPort.Write(((char)9).ToString());
                System.Threading.Thread.Sleep(600);

                foreach (char c in cfg)
                {
                    serialPort.Write(c.ToString());
                    System.Threading.Thread.Sleep(5);
                }

                serialPort.Write("=");
                System.Threading.Thread.Sleep(5);

                foreach (char c in value)
                {
                    serialPort.Write(c.ToString());
                    System.Threading.Thread.Sleep(5);
                }

                // Command stop configuration mode
                serialPort.Write(((char)13).ToString());
                System.Threading.Thread.Sleep(500);
            }
            catch (Exception)
            {

            }
            finally
            {
                configMode = false;
            }
        }
    }
}
