﻿// Copyright (c) 2020, Identix identix@identix.com.br
// 
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using IRI.Win32.CSharp.IRIWrapper;

namespace Identix.SerialSDK.Examples.WriteMultipleEpc
{
    /// <summary>
    ///  IRI Wrapper example on how to write multiple EPC.
    /// </summary>
    class Program
    {
        private static List<EpcWrite> tagsToWrite;
        private static DateTime startProcess, startWrite, endProcess, endWrite;
        private static string comPort;

        static void Main(string[] args)
        {
            Console.Write("Inform the COM port number: ");
            comPort = Console.ReadLine();
            Console.WriteLine("COM{0}", comPort);

            startProcess = DateTime.Now;

            // Create tag data
            tagsToWrite = new List<EpcWrite>();
            tagsToWrite.Add(new EpcWrite("000000000000000000000001", "000000000000000000000011"));
            tagsToWrite.Add(new EpcWrite("000000000000000000000002", "000000000000000000000012"));

            //tagsToWrite.Add(new EpcWrite("000000000000000000000011", "000000000000000000000001"));
            //tagsToWrite.Add(new EpcWrite("000000000000000000000012", "000000000000000000000002"));            

            // Register the callbacks
            ReaderHelper.Instance.TagReported += OnTagReported;

            // Connect to COM port, make sure to have installed compatible driver
            ReaderHelper.Instance.Connect("COM" + comPort.Trim());

            // As a good practice we will reset the reader to clean up old configurations
            ReaderHelper.Instance.SoftReset();

            // Create settings using default settings
            ReaderSettings readerSettings = ReaderHelper.Instance.GetDefaultReaderSettings();

            // Configure regions
            readerSettings.Region = ipj_region.E_IPJ_REGION_BRAZIL_902_907_AND_915_928_MHZ;

            // Configure tag population
            readerSettings.TagPopulationEstimate = 16;

            // Configure read mode
            readerSettings.SearchMode = SearchMode.DualTarget;

            // Configure Power
            readerSettings.TxPower = 23;

            // Apply settings
            ReaderHelper.Instance.ApplyReaderSettings(readerSettings);

            WriteTags();

            // Disconnect
            ReaderHelper.Instance.Disconnect();

            Console.WriteLine("Press any key to continue...");
            Console.ReadLine();
        }

        /// <summary>
        /// Write multiple tags
        /// </summary>
        private static void WriteTags()
        {
            try
            {
                startWrite = DateTime.Now;

                // Start inventory
                ReaderHelper.Instance.Start();

                TagOpSequence tagOpSequence;
                TagOp tagOp;

                // Creating write operation for each unwritten epc
                foreach (var tagWrite in tagsToWrite.Where(x => x.Written == false))
                {
                    tagOpSequence = new TagOpSequence();
                    tagOpSequence.SequenceId = ReaderHelper.Instance.GetNextTagSequenceId();

                    tagWrite.OpId = tagOpSequence.SequenceId;

                    tagOp = new TagOp();
                    tagOp.TagOpId = ReaderHelper.Instance.GetNextTagOpId();
                    tagOpSequence.TargetEpc = tagWrite.EpcTarget;

                    tagOp.TagOperationType = ipj_tag_operation_type.E_IPJ_TAG_OPERATION_TYPE_WRITE_EPC;
                    tagOp.Bank = ipj_mem_bank.E_IPJ_MEM_BANK_EPC;
                    tagOp.AccessPassword = "00000000";

                    tagOp.TagData = UHFTagData.FromHexString(tagWrite.NewEpc);

                    tagOpSequence.TagOps.Add(tagOp);

                    ReaderHelper.Instance.AddTagOpSequence(tagOpSequence);
                }

                // Writing time
                Thread.Sleep(1000);

                // Stop inventory
                ReaderHelper.Instance.Stop();

                // Start inventory and set time in milliseconds.
                // Inventory stops automatically right after the set time
                ReaderHelper.Instance.Start(500);

                endWrite = DateTime.Now;
                endProcess = DateTime.Now;

                TimeSpan tsProcess = endProcess - startProcess;
                TimeSpan tsWrite = endWrite - startWrite;

                Console.WriteLine("Time Write {0}", tsWrite);
                Console.WriteLine("Time Process {0}", tsProcess);

                WriteStatus();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Write tag status
        /// </summary>
        private static void WriteStatus()
        {
            foreach (var tagWrite in tagsToWrite)
            {
                Console.WriteLine("{0} => {1} : {2}", tagWrite.EpcTarget, tagWrite.NewEpc, (tagWrite.Written ? "written" : "unwritten"));
            }
        }

        // The tag read handler
        private static void OnTagReported(Tag tagReported)
        {
            if (tagsToWrite.Count > 0)
            {// Writing operation in progress
                var tag = tagsToWrite.Where(x => x.NewEpc == tagReported.Epc).FirstOrDefault();
                if (tag != null)
                    tag.Written = true;
            }
        }
    }
}
